@@ -1,8 +1,7 @@
 /**************************************************************************
  *   chars.c  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,  *
- *   2010, 2011, 2013, 2014 Free Software Foundation, Inc.                *
+ *   Copyright (C) 2001-2011, 2013-2017 Free Software Foundation, Inc.    *
  *   Copyright (C) 2016 Benno Schulenberg                                 *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -26,12 +25,8 @@
 #include <ctype.h>
 
 #ifdef ENABLE_UTF8
-#ifdef HAVE_WCHAR_H
 #include <wchar.h>
-#endif
-#ifdef HAVE_WCTYPE_H
 #include <wctype.h>
-#endif
 
 static bool use_utf8 = FALSE;
 	/* Whether we've enabled UTF-8 support. */
@@ -61,22 +56,6 @@ char *addstrings(char* str1, size_t len1, char* str2, size_t len2)
     return str1;
 }
 
-#ifndef HAVE_ISBLANK
-/* This function is equivalent to isblank(). */
-bool nisblank(int c)
-{
-    return isspace(c) && (c == '\t' || !is_cntrl_char(c));
-}
-#endif
-
-#if !defined(HAVE_ISWBLANK) && defined(ENABLE_UTF8)
-/* This function is equivalent to iswblank(). */
-bool niswblank(wchar_t wc)
-{
-    return iswspace(wc) && (wc == '\t' || !is_cntrl_wchar(wc));
-}
-#endif
-
 /* Return TRUE if the value of c is in byte range, and FALSE otherwise. */
 bool is_byte(int c)
 {
@@ -97,7 +76,7 @@ bool is_alpha_mbchar(const char *c)
     if (use_utf8) {
 	wchar_t wc;
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
+	if (mbtowc(&wc, c, MAXCHARLEN) < 0) {
 	    mbtowc_reset();
 	    return 0;
 	}
@@ -117,7 +96,7 @@ bool is_alnum_mbchar(const char *c)
     if (use_utf8) {
 	wchar_t wc;
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
+	if (mbtowc(&wc, c, MAXCHARLEN) < 0) {
 	    mbtowc_reset();
 	    return 0;
 	}
@@ -137,7 +116,7 @@ bool is_blank_mbchar(const char *c)
     if (use_utf8) {
 	wchar_t wc;
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
+	if (mbtowc(&wc, c, MAXCHARLEN) < 0) {
 	    mbtowc_reset();
 	    return 0;
 	}
@@ -185,7 +164,7 @@ bool is_punct_mbchar(const char *c)
     if (use_utf8) {
 	wchar_t wc;
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
+	if (mbtowc(&wc, c, MAXCHARLEN) < 0) {
 	    mbtowc_reset();
 	    return 0;
 	}
@@ -208,7 +187,7 @@ bool is_word_mbchar(const char *c, bool allow_punct)
 	return TRUE;
 
     if (word_chars != NULL && *word_chars != '\0') {
-	char symbol[mb_cur_max() + 1];
+	char symbol[MAXCHARLEN + 1];
 	int symlen = parse_mbchar(c, symbol, NULL);
 
 	symbol[symlen] = '\0';
@@ -260,7 +239,7 @@ int length_of_char(const char *c, int *width)
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	wchar_t wc;
-	int charlen = mbtowc(&wc, c, MB_CUR_MAX);
+	int charlen = mbtowc(&wc, c, MAXCHARLEN);
 
 	/* If the sequence is invalid... */
 	if (charlen < 0) {
@@ -293,7 +272,7 @@ int mbwidth(const char *c)
 	wchar_t wc;
 	int width;
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
+	if (mbtowc(&wc, c, MAXCHARLEN) < 0) {
 	    mbtowc_reset();
 	    return 1;
 	}
@@ -309,17 +288,6 @@ int mbwidth(const char *c)
 	return 1;
 }
 
-/* Return the maximum length (in bytes) of a character. */
-int mb_cur_max(void)
-{
-#ifdef ENABLE_UTF8
-    if (use_utf8)
-	return MB_CUR_MAX;
-    else
-#endif
-	return 1;
-}
-
 /* Convert the Unicode value in chr to a multibyte character, if possible.
  * If the conversion succeeds, return the (dynamically allocated) multibyte
  * character and its length.  Otherwise, return an undefined (dynamically
@@ -330,7 +298,7 @@ char *make_mbchar(long chr, int *chr_mb_len)
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	chr_mb = charalloc(MB_CUR_MAX);
+	chr_mb = charalloc(MAXCHARLEN);
 	*chr_mb_len = wctomb(chr_mb, (wchar_t)chr);
 
 	/* Reject invalid Unicode characters. */
@@ -360,7 +328,7 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	/* Get the number of bytes in the multibyte character. */
-	length = mblen(buf, MB_CUR_MAX);
+	length = mblen(buf, MAXCHARLEN);
 
 	/* When the multibyte sequence is invalid, only take the first byte. */
 	if (length <= 0) {
@@ -430,10 +398,10 @@ size_t move_mbleft(const char *buf, size_t pos)
     /* There is no library function to move backward one multibyte
      * character.  So we just start groping for one at the farthest
      * possible point. */
-    if (mb_cur_max() > pos)
+    if (pos < MAXCHARLEN)
 	before = 0;
     else
-	before = pos - mb_cur_max();
+	before = pos - MAXCHARLEN;
 
     while (before < pos) {
 	char_len = parse_mbchar(buf + before, NULL, NULL);
@@ -450,36 +418,12 @@ size_t move_mbright(const char *buf, size_t pos)
     return pos + parse_mbchar(buf + pos, NULL, NULL);
 }
 
-#ifndef HAVE_STRCASECMP
-/* This function is equivalent to strcasecmp(). */
-int nstrcasecmp(const char *s1, const char *s2)
-{
-    return strncasecmp(s1, s2, HIGHEST_POSITIVE);
-}
-#endif
-
 /* This function is equivalent to strcasecmp() for multibyte strings. */
 int mbstrcasecmp(const char *s1, const char *s2)
 {
     return mbstrncasecmp(s1, s2, HIGHEST_POSITIVE);
 }
 
-#ifndef HAVE_STRNCASECMP
-/* This function is equivalent to strncasecmp(). */
-int nstrncasecmp(const char *s1, const char *s2, size_t n)
-{
-    if (s1 == s2)
-	return 0;
-
-    for (; *s1 != '\0' && *s2 != '\0' && n > 0; s1++, s2++, n--) {
-	if (tolower(*s1) != tolower(*s2))
-	    break;
-    }
-
-    return (n > 0) ? tolower(*s1) - tolower(*s2) : 0;
-}
-#endif
-
 /* This function is equivalent to strncasecmp() for multibyte strings. */
 int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 {
@@ -490,12 +434,12 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	while (*s1 != '\0' && *s2 != '\0' && n > 0) {
 	    bool bad1 = FALSE, bad2 = FALSE;
 
-	    if (mbtowc(&wc1, s1, MB_CUR_MAX) < 0) {
+	    if (mbtowc(&wc1, s1, MAXCHARLEN) < 0) {
 		mbtowc_reset();
 		bad1 = TRUE;
 	    }
 
-	    if (mbtowc(&wc2, s2, MB_CUR_MAX) < 0) {
+	    if (mbtowc(&wc2, s2, MAXCHARLEN) < 0) {
 		mbtowc_reset();
 		bad2 = TRUE;
 	    }
@@ -524,28 +468,6 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	return strncasecmp(s1, s2, n);
 }
 
-#ifndef HAVE_STRCASESTR
-/* This function is equivalent to strcasestr(). */
-char *nstrcasestr(const char *haystack, const char *needle)
-{
-    size_t needle_len;
-
-    if (*needle == '\0')
-	return (char *)haystack;
-
-    needle_len = strlen(needle);
-
-    while (*haystack != '\0') {
-	if (strncasecmp(haystack, needle, needle_len) == 0)
-	    return (char *)haystack;
-
-	haystack++;
-    }
-
-    return NULL;
-}
-#endif
-
 /* This function is equivalent to strcasestr() for multibyte strings. */
 char *mbstrcasestr(const char *haystack, const char *needle)
 {
@@ -665,19 +587,6 @@ size_t mbstrlen(const char *s)
     return mbstrnlen(s, (size_t)-1);
 }
 
-#ifndef HAVE_STRNLEN
-/* This function is equivalent to strnlen(). */
-size_t nstrnlen(const char *s, size_t maxlen)
-{
-    size_t n = 0;
-
-    for (; *s != '\0' && maxlen > 0; s++, maxlen--, n++)
-	;
-
-    return n;
-}
-#endif
-
 /* This function is equivalent to strnlen() for multibyte strings. */
 size_t mbstrnlen(const char *s, size_t maxlen)
 {
@@ -704,11 +613,11 @@ char *mbstrchr(const char *s, const char *c)
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	bool bad_s_mb = FALSE, bad_c_mb = FALSE;
-	char symbol[MB_CUR_MAX];
+	char symbol[MAXCHARLEN];
 	const char *q = s;
 	wchar_t ws, wc;
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
+	if (mbtowc(&wc, c, MAXCHARLEN) < 0) {
 	    mbtowc_reset();
 	    wc = (unsigned char)*c;
 	    bad_c_mb = TRUE;
@@ -815,7 +724,7 @@ char *mbrevstrpbrk(const char *s, const char *accept, const char
 bool has_blank_chars(const char *s)
 {
     for (; *s != '\0'; s++) {
-	if (isblank(*s))
+	if (isblank((unsigned char)*s))
 	    return TRUE;
     }
 
@@ -828,7 +737,7 @@ bool has_blank_mbchars(const char *s)
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char symbol[MB_CUR_MAX];
+	char symbol[MAXCHARLEN];
 
 	for (; *s != '\0'; s += move_mbright(s, 0)) {
 	    parse_mbchar(s, symbol, NULL);
