@@ -1,9 +1,8 @@
 /**************************************************************************
  *   search.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,  *
- *   2008, 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.    *
- *   Copyright (C) 2015, 2016 Benno Schulenberg                           *
+ *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2015, 2016, 2017 Benno Schulenberg                     *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -35,26 +34,22 @@ static bool came_full_circle = FALSE;
 static bool history_changed = FALSE;
 	/* Have any of the history lists changed? */
 #endif
-#ifdef HAVE_REGEX_H
 static bool regexp_compiled = FALSE;
 	/* Have we compiled any regular expressions? */
 
-/* Compile the regular expression regexp to see if it's valid.  Return
- * TRUE if it is, or FALSE otherwise. */
+/* Compile the given regular expression and store it in search_regexp.
+ * Return TRUE if the expression is valid, and FALSE otherwise. */
 bool regexp_init(const char *regexp)
 {
-    int rc;
-
-    assert(!regexp_compiled);
-
-    rc = regcomp(&search_regexp, fixbounds(regexp),
+    int value = regcomp(&search_regexp, fixbounds(regexp),
 		NANO_REG_EXTENDED | (ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE));
 
-    if (rc != 0) {
-	size_t len = regerror(rc, &search_regexp, NULL, 0);
+    /* If regex compilation failed, show the error message. */
+    if (value != 0) {
+	size_t len = regerror(value, &search_regexp, NULL, 0);
 	char *str = charalloc(len);
 
-	regerror(rc, &search_regexp, str, len);
+	regerror(value, &search_regexp, str, len);
 	statusline(ALERT, _("Bad regex \"%s\": %s"), regexp, str);
 	free(str);
 
@@ -75,7 +70,6 @@ void regexp_cleanup(void)
 	regfree(&search_regexp);
     }
 }
-#endif
 
 /* Indicate on the statusbar that the string at str was not found by the
  * last search. */
@@ -105,9 +99,7 @@ void search_replace_abort(void)
     if (openfile->mark_set)
 	refresh_needed = TRUE;
 #endif
-#ifdef HAVE_REGEX_H
     regexp_cleanup();
-#endif
 }
 
 /* Set up the system variables for a search or replace.  If use_answer
@@ -150,10 +142,7 @@ int search_init(bool replacing, bool use_answer)
 	buf = mallocstrcpy(NULL, "");
 
     /* This is now one simple call.  It just does a lot. */
-    i = do_prompt(FALSE,
-#ifndef DISABLE_TABCOMP
-		TRUE,
-#endif
+    i = do_prompt(FALSE, FALSE,
 		replacing ? MREPLACE : MWHEREIS, backupstring,
 #ifndef DISABLE_HISTORIES
 		&search_history,
@@ -162,10 +151,7 @@ int search_init(bool replacing, bool use_answer)
 		edit_refresh, "%s%s%s%s%s%s", _("Search"),
 		/* TRANSLATORS: The next three modify the search prompt. */
 		ISSET(CASE_SENSITIVE) ? _(" [Case Sensitive]") : "",
-#ifdef HAVE_REGEX_H
-		ISSET(USE_REGEXP) ? _(" [Regexp]") :
-#endif
-		"",
+		ISSET(USE_REGEXP) ? _(" [Regexp]") : "",
 		ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") : "", replacing ?
 #ifndef NANO_TINY
 		/* TRANSLATORS: The next two modify the search prompt. */
@@ -195,11 +181,9 @@ int search_init(bool replacing, bool use_answer)
 	    update_history(&search_history, answer);
 #endif
 	}
-#ifdef HAVE_REGEX_H
 	if (ISSET(USE_REGEXP) && !regexp_init(last_search))
 	    return -1;
 	else
-#endif
 	    return 0;	/* We have a valid string or regex. */
     }
 
@@ -213,15 +197,11 @@ int search_init(bool replacing, bool use_answer)
 	TOGGLE(BACKWARDS_SEARCH);
 	backupstring = mallocstrcpy(backupstring, answer);
 	return 1;
-    } else
-#ifdef HAVE_REGEX_H
-    if (func == regexp_void) {
+    } else if (func == regexp_void) {
 	TOGGLE(USE_REGEXP);
 	backupstring = mallocstrcpy(backupstring, answer);
 	return 1;
-    } else
-#endif
-    if (func == do_replace || func == flip_replace_void) {
+    } else if (func == do_replace || func == flip_replace_void) {
 	backupstring = mallocstrcpy(backupstring, answer);
 	return -2;	/* Call the opposite search function. */
     } else if (func == do_gotolinecolumn_void) {
@@ -237,32 +217,23 @@ int search_init(bool replacing, bool use_answer)
  * where we first started searching, at column begin_x.  Return 1 when we
  * found something, 0 when nothing, and -2 on cancel.  When match_len is
  * not NULL, set it to the length of the found string, if any. */
-int findnextstr(const char *needle, bool whole_word_only, size_t *match_len,
-	const filestruct *begin, size_t begin_x)
+int findnextstr(const char *needle, bool whole_word_only, bool have_region,
+	size_t *match_len, bool skipone, const filestruct *begin, size_t begin_x)
 {
-    size_t found_len;
-	/* The length of the match we find. */
+    size_t found_len = strlen(needle);
+	/* The length of a match -- will be recomputed for a regex. */
     int feedback = 0;
 	/* When bigger than zero, show and wipe the "Searching..." message. */
-    filestruct *fileptr = openfile->current;
-    const char *rev_start = fileptr->data, *found = NULL;
+    filestruct *line = openfile->current;
+	/* The line that we will search through now. */
+    const char *from = line->data + openfile->current_x;
+	/* The point in the line from where we start searching. */
+    const char *found = NULL;
+	/* A pointer to the location of the match, if any. */
     size_t found_x;
 	/* The x coordinate of a found occurrence. */
     time_t lastkbcheck = time(NULL);
 
-    /* rev_start might end up 1 character before the start or after the
-     * end of the line.  This won't be a problem because strstrwrapper()
-     * will return immediately and say that no match was found, and
-     * rev_start will be properly set when the search continues on the
-     * previous or next line. */
-    if (ISSET(BACKWARDS_SEARCH)) {
-	if (openfile->current_x == 0)
-	    rev_start += -1;
-	else
-	    rev_start += move_mbleft(fileptr->data, openfile->current_x);
-    } else
-	rev_start += move_mbright(fileptr->data, openfile->current_x);
-
     enable_nodelay();
 
     if (begin == NULL)
@@ -293,32 +264,39 @@ int findnextstr(const char *needle, bool whole_word_only, size_t *match_len,
 	}
 
 	/* Search for the needle in the current line. */
-	found = strstrwrapper(fileptr->data, needle, rev_start);
+	if (!skipone)
+	    found = strstrwrapper(line->data, needle, from);
+
+	/* Ignore the initial match at the starting position: continue
+	 * searching from the next character, or invalidate the match. */
+	if (skipone || (!whole_word_only && !came_full_circle &&
+				found == begin->data + begin_x)) {
+	    skipone = FALSE;
+	    if (ISSET(BACKWARDS_SEARCH) && from != line->data) {
+		from = line->data + move_mbleft(line->data, from - line->data);
+		continue;
+	    } else if (!ISSET(BACKWARDS_SEARCH) && *from != '\0') {
+		from += move_mbright(from, 0);
+		continue;
+	    }
+	    found = NULL;
+	}
 
 	if (found != NULL) {
-	    /* Remember the length of the potential match. */
-	    found_len =
-#ifdef HAVE_REGEX_H
-		ISSET(USE_REGEXP) ?
-		regmatches[0].rm_eo - regmatches[0].rm_so :
-#endif
-		strlen(needle);
-
+	    /* When doing a regex search, compute the length of the match. */
+	    if (ISSET(USE_REGEXP))
+		found_len = regmatches[0].rm_eo - regmatches[0].rm_so;
 #ifndef DISABLE_SPELLER
-	    /* When we're spell checking, a match is only a true match when
-	     * it is a separate word. */
-	    if (whole_word_only) {
-		if (is_separate_word(found - fileptr->data, found_len,
-					fileptr->data))
-		    break;
-		else {
-		    /* Maybe there is a whole word in the rest of the line. */
-		    rev_start = found + 1;
-		    continue;
-		}
-	    } else
+	    /* When we're spell checking, a match should be a separate word;
+	     * if it's not, continue looking in the rest of the line. */
+	    if (whole_word_only && !is_separate_word(found - line->data,
+						found_len, line->data)) {
+		from = found + move_mbright(found, 0);
+		continue;
+	    }
 #endif
-		break;
+	    /* The match is valid. */
+	    break;
 	}
 
 	/* If we're back at the beginning, then there is no needle. */
@@ -330,23 +308,22 @@ int findnextstr(const char *needle, bool whole_word_only, size_t *match_len,
 
 	/* Move to the previous or next line in the file. */
 	if (ISSET(BACKWARDS_SEARCH))
-	    fileptr = fileptr->prev;
+	    line = line->prev;
 	else
-	    fileptr = fileptr->next;
+	    line = line->next;
 
-	/* If we've reached the start or end of the buffer, wrap around. */
-	if (fileptr == NULL) {
-#ifndef DISABLE_SPELLER
-	    /* When we're spell-checking, end-of-buffer means we're done. */
-	    if (whole_word_only) {
+	/* If we've reached the start or end of the buffer, wrap around;
+	 * but stop when spell-checking or replacing in a region. */
+	if (line == NULL) {
+	    if (whole_word_only || have_region) {
 		disable_nodelay();
 		return 0;
 	    }
-#endif
+
 	    if (ISSET(BACKWARDS_SEARCH))
-		fileptr = openfile->filebot;
+		line = openfile->filebot;
 	    else
-		fileptr = openfile->fileage;
+		line = openfile->fileage;
 
 	    statusbar(_("Search Wrapped"));
 	    /* Delay the "Searching..." message for at least two seconds. */
@@ -354,16 +331,16 @@ int findnextstr(const char *needle, bool whole_word_only, size_t *match_len,
 	}
 
 	/* If we've reached the original starting line, take note. */
-	if (fileptr == begin)
+	if (line == begin)
 	    came_full_circle = TRUE;
 
 	/* Set the starting x to the start or end of the line. */
-	rev_start = fileptr->data;
+	from = line->data;
 	if (ISSET(BACKWARDS_SEARCH))
-	    rev_start += strlen(fileptr->data);
+	    from += strlen(line->data);
     }
 
-    found_x = found - fileptr->data;
+    found_x = found - line->data;
 
     /* Ensure that the found occurrence is not beyond the starting x. */
     if (came_full_circle && ((!ISSET(BACKWARDS_SEARCH) && found_x > begin_x) ||
@@ -376,9 +353,8 @@ int findnextstr(const char *needle, bool whole_word_only, size_t *match_len,
     disable_nodelay();
 
     /* Set the current position to point at what we found. */
-    openfile->current = fileptr;
+    openfile->current = line;
     openfile->current_x = found_x;
-    openfile->current_y = fileptr->lineno - openfile->edittop->lineno;
 
     /* When requested, pass back the length of the match. */
     if (match_len != NULL)
@@ -446,10 +422,8 @@ void do_research(void)
 	return;
     }
 
-#ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP) && !regexp_init(last_search))
 	return;
-#endif
 
     /* Use the search-menu key bindings, to allow cancelling. */
     currmenu = MWHEREIS;
@@ -470,7 +444,7 @@ void go_looking(void)
 
     came_full_circle = FALSE;
 
-    didfind = findnextstr(last_search, FALSE, NULL,
+    didfind = findnextstr(last_search, FALSE, FALSE, NULL, FALSE,
 				openfile->current, openfile->current_x);
 
     /* If we found something, and we're back at the exact same spot
@@ -487,7 +461,6 @@ void go_looking(void)
     search_replace_abort();
 }
 
-#ifdef HAVE_REGEX_H
 /* Calculate the size of the replacement text, taking possible
  * subexpressions \1 to \9 into account.  Return the replacement
  * text in the passed string only when create is TRUE. */
@@ -519,7 +492,7 @@ int replace_regexp(char *string, bool create)
 	     * subexpression match to the new line. */
 	    if (create) {
 		strncpy(string, openfile->current->data +
-			openfile->current_x + regmatches[num].rm_so, i);
+					regmatches[num].rm_so, i);
 		string += i;
 	    }
 	}
@@ -530,7 +503,6 @@ int replace_regexp(char *string, bool create)
 
     return replacement_size;
 }
-#endif /* HAVE_REGEX_H */
 
 /* Return a copy of the current line with one needle replaced. */
 char *replace_line(const char *needle)
@@ -540,17 +512,13 @@ char *replace_line(const char *needle)
     size_t new_line_size = strlen(openfile->current->data) + 1;
 
     /* First adjust the size of the new line for the change. */
-#ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
 	match_len = regmatches[0].rm_eo - regmatches[0].rm_so;
 	new_line_size += replace_regexp(NULL, FALSE) - match_len;
     } else {
-#endif
 	match_len = strlen(needle);
 	new_line_size += strlen(answer) - match_len;
-#ifdef HAVE_REGEX_H
     }
-#endif
 
     /* Create the buffer. */
     copy = charalloc(new_line_size);
@@ -559,11 +527,9 @@ char *replace_line(const char *needle)
     strncpy(copy, openfile->current->data, openfile->current_x);
 
     /* Add the replacement text. */
-#ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP))
 	replace_regexp(copy + openfile->current_x, TRUE);
     else
-#endif
 	strcpy(copy + openfile->current_x, answer);
 
     assert(openfile->current_x + match_len <= strlen(openfile->current->data));
@@ -586,16 +552,19 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
     ssize_t numreplaced = -1;
     size_t match_len;
     bool replaceall = FALSE;
+    bool skipone = FALSE;
+    bool mark_was_set = FALSE;
 #ifndef NANO_TINY
-    bool old_mark_set = openfile->mark_set;
     filestruct *top, *bot;
     size_t top_x, bot_x;
     bool right_side_up = FALSE;
 	/* TRUE if (mark_begin, mark_begin_x) is the top of the mark,
 	 * FALSE if (current, current_x) is. */
 
+    mark_was_set = openfile->mark_set;
+
     /* If the mark is on, frame the region, and turn the mark off. */
-    if (old_mark_set) {
+    if (mark_was_set) {
 	mark_order((const filestruct **)&top, &top_x,
 			(const filestruct **)&bot, &bot_x, &right_side_up);
 	openfile->mark_set = FALSE;
@@ -603,7 +572,7 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 	/* Start either at the top or the bottom of the marked region. */
 	if (!ISSET(BACKWARDS_SEARCH)) {
 	    openfile->current = top;
-	    openfile->current_x = (top_x == 0 ? 0 : top_x - 1);
+	    openfile->current_x = top_x;
 	} else {
 	    openfile->current = bot;
 	    openfile->current_x = bot_x;
@@ -615,8 +584,8 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 
     while (TRUE) {
 	int i = 0;
-	int result = findnextstr(needle, whole_word_only, &match_len,
-					real_current, *real_current_x);
+	int result = findnextstr(needle, whole_word_only, mark_was_set,
+			&match_len, skipone, real_current, *real_current_x);
 
 	/* If nothing more was found, or the user aborted, stop looping. */
 	if (result < 1) {
@@ -626,15 +595,14 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 	}
 
 #ifndef NANO_TINY
-	if (old_mark_set) {
-	    /* When we've found an occurrence outside of the marked region,
-	     * stop the fanfare. */
-	    if (openfile->current->lineno > bot->lineno ||
-			openfile->current->lineno < top->lineno ||
-			(openfile->current == bot && openfile->current_x > bot_x) ||
-			(openfile->current == top && openfile->current_x < top_x))
-		break;
-	}
+	/* An occurrence outside of the marked region means we're done. */
+	if (mark_was_set && (openfile->current->lineno > bot->lineno ||
+				openfile->current->lineno < top->lineno ||
+				(openfile->current == bot &&
+				openfile->current_x + match_len > bot_x) ||
+				(openfile->current == top &&
+				openfile->current_x < top_x)))
+	    break;
 #endif
 
 	/* Indicate that we found the search string. */
@@ -666,6 +634,10 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 		break;
 	    else if (i == 2)
 		replaceall = TRUE;
+
+	    /* When "No" or moving backwards, the search routine should
+	     * first move one character further before continuing. */
+	    skipone = (i == 0 || ISSET(BACKWARDS_SEARCH));
 	}
 
 	if (i == 1 || replaceall) {  /* Yes, replace it. */
@@ -682,7 +654,7 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 #ifndef NANO_TINY
 	    /* If the mark was on and it was located after the cursor,
 	     * then adjust its x position for any text length changes. */
-	    if (old_mark_set && !right_side_up) {
+	    if (mark_was_set && !right_side_up) {
 		if (openfile->current == openfile->mark_begin &&
 			openfile->mark_begin_x > openfile->current_x) {
 		    if (openfile->mark_begin_x < openfile->current_x + match_len)
@@ -695,10 +667,10 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 
 	    /* If the mark was not on or it was before the cursor, then
 	     * adjust the cursor's x position for any text length changes. */
-	    if (!old_mark_set || right_side_up) {
+	    if (!mark_was_set || right_side_up) {
 #endif
 		if (openfile->current == real_current &&
-			openfile->current_x <= *real_current_x) {
+			openfile->current_x < *real_current_x) {
 		    if (*real_current_x < openfile->current_x + match_len)
 			*real_current_x = openfile->current_x + match_len;
 		    *real_current_x += length_change;
@@ -708,29 +680,20 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 #endif
 	    }
 
-#ifdef HAVE_REGEX_H
 	    /* Don't find the same zero-length or BOL match again. */
 	    if (match_len == 0 || (*needle == '^' && ISSET(USE_REGEXP)))
-		match_len++;
-#endif
-	    /* Set the cursor at the last character of the replacement
-	     * text, so that searching will continue /after/ it.  Note
-	     * that current_x might be set to (size_t)-1 here. */
+		skipone = TRUE;
+
+	    /* When moving forward, put the cursor just after the replacement
+	     * text, so that searching will continue there. */
 	    if (!ISSET(BACKWARDS_SEARCH))
-		openfile->current_x += match_len + length_change - 1;
+		openfile->current_x += match_len + length_change;
 
 	    /* Update the file size, and put the changed line into place. */
 	    openfile->totsize += mbstrlen(copy) - mbstrlen(openfile->current->data);
 	    free(openfile->current->data);
 	    openfile->current->data = copy;
 
-#ifndef DISABLE_COLOR
-	    /* Reset the precalculated multiline-regex hints only when
-	     * the first replacement has been made. */
-	    if (numreplaced == 0)
-		reset_multis(openfile->current, TRUE);
-#endif
-
 	    if (!replaceall) {
 #ifndef DISABLE_COLOR
 		/* When doing syntax coloring, the replacement might require
@@ -750,9 +713,13 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 
     if (numreplaced == -1)
 	not_found_msg(needle);
+#ifndef DISABLE_COLOR
+    else if (numreplaced > 0)
+	refresh_needed = TRUE;
+#endif
 
 #ifndef NANO_TINY
-    if (old_mark_set)
+    if (mark_was_set)
 	openfile->mark_set = TRUE;
 #endif
 
@@ -768,7 +735,7 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 void do_replace(void)
 {
     filestruct *edittop_save, *begin;
-    size_t begin_x;
+    size_t firstcolumn_save, begin_x;
     ssize_t numreplaced;
     int i;
 
@@ -789,11 +756,7 @@ void do_replace(void)
     if (i != 0)
 	return;
 
-    i = do_prompt(FALSE,
-#ifndef DISABLE_TABCOMP
-		TRUE,
-#endif
-		MREPLACEWITH, NULL,
+    i = do_prompt(FALSE, FALSE, MREPLACEWITH, NULL,
 #ifndef DISABLE_HISTORIES
 		&replace_history,
 #endif
@@ -816,6 +779,7 @@ void do_replace(void)
 
     /* Save where we are. */
     edittop_save = openfile->edittop;
+    firstcolumn_save = openfile->firstcolumn;
     begin = openfile->current;
     begin_x = openfile->current_x;
 
@@ -823,6 +787,7 @@ void do_replace(void)
 
     /* Restore where we were. */
     openfile->edittop = edittop_save;
+    openfile->firstcolumn = firstcolumn_save;
     openfile->current = begin;
     openfile->current_x = begin_x;
     refresh_needed = TRUE;
@@ -858,11 +823,8 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	functionptrtype func;
 
 	/* Ask for the line and column. */
-	int i = do_prompt(FALSE,
-#ifndef DISABLE_TABCOMP
-		TRUE,
-#endif
-		MGOTOLINE, use_answer ? answer : NULL,
+	int i = do_prompt(FALSE, FALSE, MGOTOLINE,
+		use_answer ? answer : NULL,
 #ifndef DISABLE_HISTORIES
 		NULL,
 #endif
@@ -889,7 +851,7 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 
 	/* Try to extract one or two numbers from the user's response. */
 	if (!parse_line_column(answer, &line, &column)) {
-	    statusbar(_("Invalid line or column number"));
+	    statusline(ALERT, _("Invalid line or column number"));
 	    return;
 	}
     } else {
@@ -922,17 +884,29 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     openfile->placewewant = column - 1;
 
     /* When the position was manually given, center the target line. */
-    if (interactive || ISSET(SOFTWRAP)) {
+    if (interactive) {
 	adjust_viewport(CENTERING);
 	refresh_needed = TRUE;
     } else {
+	int rows_from_tail;
+
+#ifndef NANO_TINY
+	if (ISSET(SOFTWRAP)) {
+	    filestruct *line = openfile->current;
+	    size_t leftedge = (xplustabs() / editwincols) * editwincols;
+
+	    rows_from_tail = (editwinrows / 2) -
+			go_forward_chunks(editwinrows / 2, &line, &leftedge);
+	} else
+#endif
+	    rows_from_tail = openfile->filebot->lineno -
+				openfile->current->lineno;
+
 	/* If the target line is close to the tail of the file, put the last
-	 * line of the file on the bottom line of the screen; otherwise, just
+	 * line or chunk on the bottom line of the screen; otherwise, just
 	 * center the target line. */
-	if (openfile->filebot->lineno - openfile->current->lineno <
-							editwinrows / 2) {
-	    openfile->current_y = editwinrows - openfile->filebot->lineno +
-					openfile->current->lineno - 1;
+	if (rows_from_tail < editwinrows / 2) {
+	    openfile->current_y = editwinrows - 1 - rows_from_tail;
 	    adjust_viewport(STATIONARY);
 	} else
 	    adjust_viewport(CENTERING);
@@ -999,7 +973,6 @@ bool find_bracket_match(bool reverse, const char *bracket_set)
     /* Set the current position to the found matching bracket. */
     openfile->current = fileptr;
     openfile->current_x = found - fileptr->data;
-    openfile->current_y = fileptr->lineno - openfile->edittop->lineno;
 
     return TRUE;
 }
@@ -1020,7 +993,7 @@ void do_find_bracket(void)
 	 * the bracket at the current cursor position. */
     int wanted_ch_len;
 	/* The length of wanted_ch in bytes. */
-    char *bracket_set;
+    char bracket_set[MAXCHARLEN * 2 + 1];
 	/* The pair of characters in ch and wanted_ch. */
     size_t i;
 	/* Generic loop variable. */
@@ -1034,8 +1007,6 @@ void do_find_bracket(void)
 	/* The initial bracket count. */
     bool reverse;
 	/* The direction we search. */
-    char *found_ch;
-	/* The character we find. */
 
     assert(mbstrlen(matchbrackets) % 2 == 0);
 
@@ -1083,20 +1054,16 @@ void do_find_bracket(void)
     wanted_ch_len = parse_mbchar(wanted_ch, NULL, NULL);
 
     /* Fill bracket_set in with the values of ch and wanted_ch. */
-    bracket_set = charalloc((mb_cur_max() * 2) + 1);
     strncpy(bracket_set, ch, ch_len);
     strncpy(bracket_set + ch_len, wanted_ch, wanted_ch_len);
-    null_at(&bracket_set, ch_len + wanted_ch_len);
-
-    found_ch = charalloc(mb_cur_max() + 1);
+    bracket_set[ch_len + wanted_ch_len] = '\0';
 
     while (TRUE) {
 	if (find_bracket_match(reverse, bracket_set)) {
 	    /* If we found an identical bracket, increment count.  If we
 	     * found a complementary bracket, decrement it. */
-	    parse_mbchar(openfile->current->data + openfile->current_x,
-			found_ch, NULL);
-	    count += (strncmp(found_ch, ch, ch_len) == 0) ? 1 : -1;
+	    count += (strncmp(openfile->current->data + openfile->current_x,
+				 ch, ch_len) == 0) ? 1 : -1;
 
 	    /* If count is zero, we've found a matching bracket.  Update
 	     * the screen and get out. */
@@ -1114,10 +1081,6 @@ void do_find_bracket(void)
 	    break;
 	}
     }
-
-    /* Clean up. */
-    free(bracket_set);
-    free(found_ch);
 }
 #endif /* !NANO_TINY */
 
